/* $Id: capi.h,v 1.2 1998/10/23 12:50:47 fritz Exp $
 *
 * This stuff is based heavily on AVM's CAPI-adk for linux.
 *
 * This program is free software; you can redistribute it and/or modify          * it under the terms of the GNU General Public License as published by          * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *                                                                               * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: capi.h,v $
 * Revision 1.2  1998/10/23 12:50:47  fritz
 * Added RCS keywords and GPL notice.
 *
 */
#ifndef _capi_h_
#define _capi_h_

extern void Handle_CAPI_Msg(void);

#endif	/* _capi_h_ */
